/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.controller;

import java.util.Collection;

import gz.cyberbrain.cyberhotel.controller.util.ErrorItem;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.model.address.to.AddressTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.AddressInUseException;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

public class AddressController {
	
	private static ConfigurationFacadeDelegate delegate = SpringFacadeFactory.getConfigurationFacadeDelegate();
	
	private AddressController() {}
	
	public static Collection countriesList() {		
		try {
			return delegate.showCountries();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static Collection countryProvincesList(String country) {		
		try {
			return delegate.showCountryProvinces(country);
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}		
		return null;
	}
	
	public static Collection provinceLocalitiesList(String country, String province) {		
		try {
			return delegate.showProvinceLocalities(country, province);
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}				
		return null;
	}
	
	public static void createAddress(String country, String province, String locality) {								
		try {
			delegate.createAddress(new AddressTo(null, country, province, locality));
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}		
	}
	
	public static void removeByCountry(String country) {
		try {
			delegate.deleteAddressByCountry(country);
		} catch (AddressInUseException e) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("delete"));
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}
	}
	
	public static void removeByProvince(String country, String province) {
	
		try {
			delegate.deleteAddressByProvince(country, province);
		} catch (AddressInUseException e) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("delete"));
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}
	}
	
	public static void removeByLocality(String country, String province, String locality) {
		try {
			delegate.deleteAddressByLocality(country, province, locality);
		} catch (AddressInUseException e) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("delete"));
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}
	}
	
	public static Long findAddressId(String country, String province, String locality) {
		
		if (country == null || province == null || locality == null) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("address"));
			return null;
		}
		
		try {
			return ((AddressTo) delegate.findByCompleteAddress(country, province, locality)).getAddressId();
		} catch (InstanceNotFoundException e) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("address"));
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}	
		return null;
	}
	
	public static String addressCountry(Long addressId) {
		try {
			return ((AddressTo) delegate.findAddress(addressId)).getCountry();
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static String addressProvince(Long addressId) {
		try {
			return ((AddressTo) delegate.findAddress(addressId)).getProvince();
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static String addressLocality(Long addressId) {
		try {
			return ((AddressTo) delegate.findAddress(addressId)).getLocality();
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}
		return null;
	}
	
}
