/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.controller.util.ErrorItem;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.model.board.to.BoardTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.BoardInUseException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.BoardTariffiedException;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class BoardController {

	private static ConfigurationFacadeDelegate delegate = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static HashMap<String, Object> attributes = new HashMap<String, Object>();
	
	private BoardController() {}
	
	public static Vector boardList() throws InternalErrorException {				
		Vector<TextItem> items = new Vector<TextItem>();
		Collection boards = delegate.showBoards();
		Iterator iter = boards.iterator();

		BoardTo board;

		for (int i=0; iter.hasNext(); i++) {

			board = (BoardTo) iter.next();
			items.add(new TextItem(board.getBoardCode(), board.getDescription()));

		}

		return items;
	}

	public static Serializable createBoard(String boardCode, String description) throws InternalErrorException {
		return delegate.createBoard(new BoardTo(boardCode, description)).getBoardCode();		
	}

	public static void updateBoard(String boardCode, String description) throws InstanceNotFoundException, InternalErrorException {				
		delegate.updateBoard(new BoardTo(boardCode, description));
	}
	
	public static void deleteBoard(String boardCode) throws BoardInUseException, BoardTariffiedException, InstanceNotFoundException, InternalErrorException {
		BoardTo board = delegate.findBoard(boardCode);
		delegate.deleteBoard(board);
	}
	
	public static HashMap findBoard(String boardCode) {
		
		BoardTo board;
		try {
			board = delegate.findBoard(boardCode);
		
		
		attributes.put("code", boardCode);
		attributes.put("description", board.getDescription());
		
		} catch (InstanceNotFoundException e) {
			ErrorsHandler.clear();
			ErrorsHandler.add(new ErrorItem("instanceNotFound"));
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}
							
		return attributes;
				
	}
	
}
