/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.model.cash.to.CashTo;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.delegate.InvoicingFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.ClosedPaymentException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.UnaceptablePaymentException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions.ClosedCashException;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.money.MoneyOperations;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class CashController {
	
	private static InvoicingFacadeDelegate delegate = SpringFacadeFactory.getInvoicingFacadeDelegate();
	private static HashMap <String, Object> attributes = new HashMap <String, Object> ();

	private CashController(){}

	public static Vector cashList() {				
		Vector <TextItem> items = new Vector <TextItem> ();
		Collection cashes;
		try {
			cashes = delegate.showCashes();

			Iterator iter = cashes.iterator();

			CashTo cash;

			for (int i=0; iter.hasNext(); i++) {

				cash = (CashTo) iter.next();
				items.add(new TextItem(cash.getId(), cash.getCashNumber().toString()));

			}

		} catch (InternalErrorException e) {
			e.printStackTrace();
		}

		return items;
	}

	public static Serializable createCash(Calendar openDate, Calendar closeDate, String status, String openUser, String closeUser)
	throws InstanceNotFoundException, InternalErrorException {

		return delegate.insertACash(new CashTo(null, openDate, closeDate, 0l, 0l, 0l, 0l, new BigDecimal(0), new BigDecimal(0), 
				new BigDecimal(0), new BigDecimal(0), status, openUser, closeUser));

	}
	
	public static void updateCash(Long cashNumber, Calendar openDate, Calendar closeDate, String status, 
			String openUser, String closeUser, Collection payments, boolean closement)
	throws ClosedCashException, UnaceptablePaymentException, ClosedPaymentException, InstanceNotFoundException, InternalErrorException {

		CashTo cashTo = delegate.findACash(cashNumber);
		
		delegate.updateACash(new CashTo(cashNumber, openDate, closeDate, cashTo.getPaymentsNumber(), cashTo.getAccountsNumber(), cashTo.getCardsNumber(), cashTo.getVouchersNumber(), cashTo.getAccountsTotal(), cashTo.getCardsTotal(), 
				cashTo.getVouchersTotal(), cashTo.getCashTotal(), status, openUser, closeUser), payments, closement);

	}

	public static HashMap findCash(Long cashNumber) throws InstanceNotFoundException, InternalErrorException {

		//Find and return the cash
		CashTo cash = delegate.findACash(cashNumber);

		attributes.put("id", cashNumber);

		attributes.put("cashNumber", cash.getCashNumber());
		attributes.put("openDate", cash.getOpenDate());
		attributes.put("closeDate", cash.getCloseDate());
		attributes.put("paymentsNumber", cash.getPaymentsNumber());
		attributes.put("accountsNumber", cash.getAccountsNumber());
		attributes.put("cardsNumber", cash.getCardsNumber());
		attributes.put("vouchersNumber", cash.getVouchersNumber());
		attributes.put("accountsTotal", MoneyOperations.getDecimalReduction(cash.getAccountsTotal()));
		attributes.put("cardsTotal", MoneyOperations.getDecimalReduction(cash.getCardsTotal()));				
		attributes.put("vouchersTotal", MoneyOperations.getDecimalReduction(cash.getVouchersTotal()));
		attributes.put("cashTotal", MoneyOperations.getDecimalReduction(cash.getCashTotal()));		
		attributes.put("status", cash.getStatus());
		attributes.put("openUser", cash.getOpenUser());
		attributes.put("closeUser", cash.getCloseUser());

		return attributes;

	}

}
