/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.model.invoicingfacade.delegate.InvoicingFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.ClosedPaymentException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.IssuedInvoiceException;
import gz.cyberbrain.cyberhotel.model.payment.to.PaymentTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.money.MoneyOperations;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.Payment;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class PaymentController {

	private static InvoicingFacadeDelegate delegate = SpringFacadeFactory.getInvoicingFacadeDelegate();
	private static HashMap <String, Object> attributes = new HashMap <String, Object> ();

	private PaymentController(){}

	public static Vector paymentList() {				
		Vector <TextItem> items = new Vector <TextItem> ();
		Collection payments;
		try {
			payments = delegate.showPayments();

			Iterator iter = payments.iterator();

			PaymentTo payment;

			for (int i=0; iter.hasNext(); i++) {

				payment = (PaymentTo) iter.next();
				items.add(new TextItem(payment.getId(), payment.getReference().toString()));

			}

		} catch (InternalErrorException e) {
			e.printStackTrace();
		}

		return items;
	}

	public static Serializable createPayment(Calendar paymentDate, BigDecimal amount, 
			String payMode, String status, String notes, Long invoiceNumber, String invoiceSerial, String user,
			Long cash, boolean issue)
	throws UnaceptableInvoiceException, IssuedInvoiceException, InstanceNotFoundException, InternalErrorException {

		return delegate.insertAPayment(new PaymentTo(null, paymentDate, amount, payMode, status, notes, 
				invoiceNumber, invoiceSerial, user, cash), issue);

	}
	public static void updatePayment(Long paymentNumber, Calendar paymentDate, BigDecimal amount, 
			String payMode, String status, String notes, Long invoiceNumber, String invoiceSerial, String user,
			Long cash)
	throws ClosedPaymentException, UnaceptableInvoiceException, IssuedInvoiceException, InstanceNotFoundException, InternalErrorException {

		delegate.updateAPayment(new PaymentTo(paymentNumber, paymentDate, amount, payMode, status, notes, 
				invoiceNumber, invoiceSerial, user, cash));

	}

	public static void deletePayment(Long reference) throws InstanceNotFoundException, InternalErrorException {

		PaymentTo payment = delegate.findAPayment(reference);
		delegate.deleteAPayment(payment);

	}

	public static HashMap findPayment(Long reference) throws InstanceNotFoundException, InternalErrorException {

		//Find and return the payment
		PaymentTo payment = delegate.findAPayment(reference);

		attributes.put("id", reference);

		attributes.put("reference", payment.getReference());
		attributes.put("paymentDate", payment.getPaymentDate());
		attributes.put("amount", MoneyOperations.getDecimalReduction(payment.getAmount()));
		attributes.put("payMode", payment.getPayMode());
		attributes.put("status", payment.getStatus());
		attributes.put("notes", payment.getNotes());
		attributes.put("invoiceNumber", payment.getInvoiceNumber());
		attributes.put("invoiceSerial", payment.getInvoiceSerial());		
		attributes.put("user", payment.getUser());
		attributes.put("cash", payment.getCash());

		return attributes;

	}

	public static Vector openedPayments() throws InternalErrorException {
		Vector<Payment> items = new Vector<Payment>();

		Collection payments = delegate.findOpenedPayments();

		Iterator iter = payments.iterator();

		PaymentTo payment;

		for (int i=0; iter.hasNext(); i++) {

			payment = (PaymentTo) iter.next();
			items.add(new Payment(payment.getReference(), payment.getPayMode(), payment.getAmount()));

		}

		return items;
	}

}
