/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.model.board.to.BoardTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.price.to.PriceTo;
import gz.cyberbrain.cyberhotel.model.pricetype.to.PriceTypeTo;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.money.MoneyOperations;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TableItem;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class PriceController {

	private static ConfigurationFacadeDelegate delegate = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static HashMap<String, Object> attributes = new HashMap<String, Object>();

	private PriceController() {}

	public static Vector priceList(Long season) throws InstanceNotFoundException,  InternalErrorException {				
		Vector<TableItem> items = new Vector<TableItem> ();
		Collection prices = delegate.showSeasonPrices(season);
		Iterator iter = prices.iterator();

		PriceTo price;
		PriceTypeTo priceType;
		BoardTo board;
		Object[] columns = new Object[5];
		Serializable[] id = new Serializable[4];

		for (int i=0; iter.hasNext(); i++) {
	
			columns = new Object[5];
			id = new Serializable[4];
			
			price = (PriceTo) iter.next();
			priceType = (PriceTypeTo) delegate.findPriceType(price.getPriceType());
			board = (BoardTo) delegate.findBoard(price.getBoard());

			columns[0] = priceType.getPriceTypeCode();
			columns[1] = priceType.getDescription();
			columns[2] = price.getBoard();
			columns[3] = board.getDescription();
			columns[4] = MoneyOperations.getDecimalReduction(price.getValue());

			id[0] = price.getSeason();
			id[1] = price.getPriceType();
			id[2] = price.getBoard();
			id[3] = price.getId();

			items.add(new TableItem(id, columns));
			
		}		

		return items;
		
	}

	public static void createPrice(Long season, String priceType, String board, Long priceId, BigDecimal value) throws InternalErrorException {
		delegate.createPrice(new PriceTo(priceId, value, board, priceType, season));
	}

	public static void updatePrice(Long season, String priceType, String board, Long priceId, BigDecimal value) throws InternalErrorException {		
		delegate.updatePrice(new PriceTo(priceId, value, board, priceType, season));
	}

	public static void deletePrice(Long season, String priceType, String board, Long priceId) throws InstanceNotFoundException,  InternalErrorException{
		PriceTo price = delegate.findPrice(priceId, board, priceType, season);
		delegate.deletePrice(price);
	}

	public static HashMap findPrice(Long season, String priceType, String board, Long priceId) throws InstanceNotFoundException, InternalErrorException {
		PriceTo price = delegate.findPrice(priceId, board, priceType, season);

		attributes.put("id", priceId);
		attributes.put("board", price.getBoard());
		attributes.put("priceType",price.getPriceType());
		attributes.put("season", price.getSeason());

		return attributes;

	}
	
	public static long maxPriceId() {
		
		long maxId = 0;
		
		try {
			maxId = delegate.maxPriceId();
		} catch (InternalErrorException e) {}
		
		return maxId;
	}

}
