/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.reservationfacade.delegate;

import gz.cyberbrain.cyberhotel.model.client.to.AgencyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ClientTo;
import gz.cyberbrain.cyberhotel.model.client.to.CompanyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ParticularTo;
import gz.cyberbrain.cyberhotel.model.owe.to.OweTo;
import gz.cyberbrain.cyberhotel.model.reservation.to.ReservationTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientPayerException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.AgencyWithReservationsException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.CancelledReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.CompanyWithReservationsException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.DifferentDayException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.FinishedReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidDocumentTypeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidPayModeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidSexException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientWithInvoicesException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ParticularWithOwesException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ParticularWithReservationsException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.PendingReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.PriceNotFoundException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RegisteredReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomNotBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAddressException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAgencyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableBankException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptableBoardException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCardException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCompanyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableConceptException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableEmployeeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableParticularException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptablePayerException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptablePriceTypeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableRoomException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnavailableRoomException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnregisteredReservationException;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;
import gz.cyberbrain.cyberhotel.model.wash.to.WashTo;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Collection;

public interface ReservationFacadeDelegate extends Serializable {	
	
	public ParticularTo insertAParticular(ParticularTo particularTo) 
	throws InvalidSexException, InvalidDocumentTypeException, InvalidPayModeException, 
	UnaceptableAddressException, UnaceptableCompanyException,
	UnaceptableBankException, UnaceptableCardException,
	InvalidIntervalException, InternalErrorException;
	
	public ParticularTo findAParticular(Long clientId) 
	throws InstanceNotFoundException, InternalErrorException;
		
	public void updateAParticular(ParticularTo particularTo) 
	throws InvalidSexException, InvalidDocumentTypeException, InvalidPayModeException, 
	UnaceptableAddressException, UnaceptableCompanyException,
	UnaceptableBankException, UnaceptableCardException,
	InvalidIntervalException, InternalErrorException;
	
	public void removeAParticular(ParticularTo particularTo) 
	throws ParticularWithOwesException, ParticularWithReservationsException, 
	ClientPayerException, ClientWithInvoicesException, InternalErrorException;
	
	public Collection showParticulars() throws InternalErrorException;
	
	public AgencyTo insertAnAgency(AgencyTo agencyTo) 
	throws InvalidPayModeException, UnaceptableAddressException, 
	UnaceptableBankException, UnaceptableCardException,
	InternalErrorException;
	
	public AgencyTo findAnAgency(Long clientId) 
	throws InstanceNotFoundException, InternalErrorException;
	
	public void updateAnAgency(AgencyTo agencyTo) 
	throws InvalidPayModeException, UnaceptableAddressException, 
	UnaceptableBankException, UnaceptableCardException,
	InternalErrorException;
	
	public void removeAnAgency(AgencyTo agencyTo) 
	throws AgencyWithReservationsException, ClientPayerException,
	ClientWithInvoicesException, InternalErrorException;
	
	public Collection showAgencies() throws InternalErrorException;
	
	public CompanyTo insertACompany(CompanyTo companyTo) 
	throws UnaceptableAddressException, UnaceptableBankException, InternalErrorException;
	
	public CompanyTo findACompany(Long clientId) 
	throws InstanceNotFoundException, InternalErrorException;
	
	public void updateACompany(CompanyTo companyTo) 
	throws UnaceptableAddressException, UnaceptableBankException, InternalErrorException;
	
	public void removeACompany(CompanyTo companyTo) 
	throws CompanyWithReservationsException, ClientPayerException,
	ClientWithInvoicesException, InternalErrorException;
	
	public Collection showCompanies() throws InternalErrorException;
	
	public ClientTo findAPayer(Long clientId) throws InstanceNotFoundException, InternalErrorException;
	
	public ReservationTo makeAReservation(ReservationTo reservationTo, Collection requestsTo) 
	throws UnaceptableParticularException, UnaceptableBoardException, UnaceptablePriceTypeException, 
	UnaceptableAgencyException, UnaceptablePayerException, UnaceptableCompanyException, UnaceptableRoomException, 
	UnavailableRoomException, InvalidIntervalException, InternalErrorException;
	
	public ReservationTo findAReservation(Long reservationId) throws InstanceNotFoundException, InternalErrorException;

	public void updateAReservation(ReservationTo reservationTo, Collection requestsTo) 
	throws UnaceptableParticularException, UnaceptableBoardException, UnaceptablePriceTypeException, 
	UnaceptableAgencyException, UnaceptablePayerException, UnaceptableCompanyException, UnaceptableRoomException, 
	UnavailableRoomException, InvalidIntervalException, InstanceNotFoundException, InternalErrorException;	
	
	public Collection showUnregisteredReservations() throws InternalErrorException;
	
	public Collection showRegisteredReservations() throws InternalErrorException;
	
	public Collection showCancelledReservations() throws InternalErrorException;
	
	public Collection showFinishedReservations() throws InternalErrorException;
	
	public Collection showReservations() throws InternalErrorException;
	
	public void removeAReservation(ReservationTo reservationTo) 
	throws RegisteredReservationException, InternalErrorException;
	
	public Collection reservationRequests(Long reservationId) throws InternalErrorException;
	
	public void checkIn(ReservationTo reservationTo) 
	throws DifferentDayException, FinishedReservationException, PriceNotFoundException, 
	CancelledReservationException, RegisteredReservationException, 
	UnaceptableRoomException, UnaceptableConceptException, RoomBusyException,
	UnaceptableParticularException, UnaceptableInvoiceException, RoomNotBusyException,
	InvalidIntervalException, InstanceNotFoundException, InternalErrorException;
	
	public ReservationTo findACheckIn (Long reservationId) 
	throws InstanceNotFoundException, InternalErrorException;
	
	public Collection findAllCheckIns() throws InternalErrorException;
	
	public void removeACheckIn(ReservationTo reservationTo) 
	throws FinishedReservationException, CancelledReservationException, UnregisteredReservationException, 
	InstanceNotFoundException, InternalErrorException;
	
	public void checkOut (ReservationTo reservationTo, Calendar trueOutgoingDate) 
	throws  DifferentDayException, PendingReservationException, PriceNotFoundException, 
	CancelledReservationException, FinishedReservationException,    
	InvalidIntervalException,UnaceptableRoomException, UnaceptableConceptException, UnaceptableParticularException, 
	UnaceptableInvoiceException, RoomNotBusyException, InstanceNotFoundException, InternalErrorException;
	
	public ReservationTo findACheckOut (Long reservationId) 
	throws InstanceNotFoundException, InternalErrorException;
	
	public Collection findAllCheckOuts() throws InternalErrorException;
	
	//public Long nextOweNumber(Long reservationId) throws InternalErrorException;
	
	public OweTo oweAddition (OweTo oweTo) 
	throws UnaceptableRoomException, UnaceptableConceptException, UnaceptableParticularException, 
	UnaceptableInvoiceException, RoomNotBusyException, InstanceNotFoundException, InternalErrorException;
	
	public void oweUpdate(OweTo oweTo) throws InternalErrorException;
	
	public OweTo findAnOwe(Long oweNumber)
	throws InstanceNotFoundException, InternalErrorException;
	
	public Collection findOwesByRoom(String roomCode) throws UnaceptableRoomException, InternalErrorException;
	
	public Collection findOwesByParticular(Long particularId) throws UnaceptableParticularException, InternalErrorException;
	
	public Collection findOwesByRoomAndParticular(String roomCode, Long particularId) 
	throws UnaceptableRoomException, UnaceptableParticularException, InternalErrorException;
	
	public void removeAnOwe (OweTo oweTo) throws InternalErrorException;
	
	public void blockARoom (RoomTo roomTo) throws RoomBusyException, InternalErrorException;
	
	public void unblockARoom (RoomTo roomTo) throws InternalErrorException;
	
	public WashTo doAWash (WashTo washTo) 
	throws UnaceptableEmployeeException, UnaceptableRoomException, InternalErrorException;
	
}
