/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions;

import java.util.Calendar;

import gz.cyberbrain.cyberhotel.model.employee.dao.EmployeeDao;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableEmployeeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableRoomException;
import gz.cyberbrain.cyberhotel.model.room.dao.RoomDao;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.wash.dao.WashDao;
import gz.cyberbrain.cyberhotel.model.wash.to.WashTo;

public class DoAWashAction {

	private WashTo washTo;
	private RoomDao roomDao;
	private EmployeeDao employeeDao;
	private WashDao washDao;

	public DoAWashAction (WashTo washTo, RoomDao roomDao, EmployeeDao employeeDao, WashDao washDao) {
		this.washTo = washTo;
		this.roomDao = roomDao;
		this.employeeDao = employeeDao;
		this.washDao = washDao;
	}

	public WashTo execute() throws UnaceptableEmployeeException, UnaceptableRoomException, InternalErrorException {

		RoomTo room;
		
		try {
			room = roomDao.find(washTo.getRoom());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableRoomException(washTo.getRoom());
		}
		
		try {
			employeeDao.find(washTo.getEmployee());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableEmployeeException(washTo.getEmployee());
		}

		//Clean the room
		
		room.setCleaningStatus("Limpa");
		
		//Disinfect the room if disinfection is checked
		
		if (washTo.isDisinfection()) room.setCheckingDate(Calendar.getInstance());
		
		//Update the room
		
		roomDao.update(room);
		
		Long washId = washDao.nextWash(washTo.getRoom(), washTo.getEmployee());
		//if (washId == null) washId = new Long(1);
		washTo.setWashId(washId);
		return washDao.create(washTo);
		
	}

}
