/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions;

import gz.cyberbrain.cyberhotel.model.address.dao.AddressDao;
import gz.cyberbrain.cyberhotel.model.bank.dao.BankDao;
import gz.cyberbrain.cyberhotel.model.card.dao.CardDao;
import gz.cyberbrain.cyberhotel.model.client.dao.ClientDao;
import gz.cyberbrain.cyberhotel.model.client.to.ParticularTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidDocumentTypeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidPayModeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidSexException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAddressException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableBankException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCardException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCompanyException;
import gz.cyberbrain.cyberhotel.model.util.dates.Interval;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;

public class UpdateAParticularAction {
	
	private ParticularTo particularTo;
	private AddressDao addressDao;
	private BankDao bankDao;
	private CardDao cardDao;
	private ClientDao companyDao;
	private ClientDao particularDao;
	
	public UpdateAParticularAction(ParticularTo particularTo, AddressDao addressDao, BankDao bankDao, CardDao cardDao, ClientDao companyDao, ClientDao particularDao) {

		this.particularTo = particularTo;
		this.addressDao = addressDao;
		this.bankDao = bankDao;
		this.cardDao = cardDao;
		this.companyDao = companyDao;
		this.particularDao = particularDao;

	}
	
	public void execute() 
	throws InvalidSexException, InvalidDocumentTypeException, InvalidPayModeException, 
	UnaceptableAddressException, UnaceptableCompanyException,
	UnaceptableBankException, UnaceptableCardException,
	InvalidIntervalException, InternalErrorException {
		
		try {
			addressDao.find(particularTo.getAddress());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableAddressException(particularTo.getAddress());
		}		
		
		try {
			if (particularTo.getBank() != null) bankDao.find(particularTo.getBank());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableBankException(particularTo.getBank());
		}
		
		try {
			if (particularTo.getCard() != null) cardDao.find(particularTo.getCard());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableCardException(particularTo.getCard());
		}
		
		try {
			if (particularTo.getCompany() != null) companyDao.find(particularTo.getCompany());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableCompanyException(particularTo.getCompany());
		}
		
		if ((particularTo.getExpeditionDate() != null) && (particularTo.getExpirationDate() != null)) { 
			@SuppressWarnings("unused")
			Interval interval = new Interval(particularTo.getExpeditionDate(), particularTo.getExpirationDate());
	}

		
		if (particularTo.getPayMode() != "Contado" 
			&& particularTo.getPayMode() != "Tarxeta" 
				&& particularTo.getPayMode() != "Bono") throw new InvalidPayModeException(particularTo.getPayMode());
		
		
		if (particularTo.getDocumentType() != "DNI" 
			&& particularTo.getDocumentType() != "Pasaporte") throw new InvalidDocumentTypeException(particularTo.getDocumentType());
		
		
		if (particularTo.getSex() != "H"
			&& particularTo.getSex() != "M") throw new InvalidSexException(particularTo.getSex());
		
		particularDao.update(particularTo);
	}

}
