/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.view.swing.edit;

import gz.cyberbrain.cyberhotel.view.swing.FillableWindow;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.ImageFileView;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.ImageFilter;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.ImagePreview;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.InfoPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;
import gz.cyberbrain.cyberhotel.controller.SaveAdapter;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;

import javax.swing.JPanel;

import java.awt.Color;
import java.awt.Image;
import java.awt.Toolkit;

import javax.swing.JLabel;
import java.awt.Rectangle; 
import java.io.File;
import java.io.Serializable;
import java.util.HashMap;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.UIManager;
import javax.swing.border.LineBorder;

public class BankingWindow extends FillableWindow {

	private static final long serialVersionUID = 1L;

	private JInternalFrame owner = null;

	private Serializable id = null;

	private String dialogTitle = null;

	private String dialogMessage = null;

	private SaveAdapter saveAdapter = null;

	private JPanel jContentPane = null;

	private JLabel nameLabel = null;

	private JTextField nameTextField = null;

	private JTextField logoTextField = null;

	private JButton exploreButton = null;

	private JLabel pathLabel = null;

	private JLabel logoLabel = null;

	private JFileChooser jFileChooser = null;

	private JButton saveButton = null;

	private JButton cancelButton = null;

	private InfoPanel infoPanel = null;

	/**
	 * @param owner
	 */
	public BankingWindow() {
		super("", false, true, false, true);
		initialize();
	}

	public void setOwner(JInternalFrame owner) {
		this.owner = owner;
	}

	public JInternalFrame getOwner() {
		return owner;
	}

	public void setDialogTitle(String dialogTitle) {
		this.dialogTitle = dialogTitle;
	}

	public void setDialogMessage(String dialogMessage) {
		this.dialogMessage = dialogMessage;
	}

	public String getDialogTitle() {
		return dialogTitle;
	}

	public String getDialogMessage() {
		return dialogMessage;
	}

	public void setAdapter(SaveAdapter saveAdapter) {
		this.saveAdapter = saveAdapter;
	}

	public SaveAdapter getAdapter() {
		return saveAdapter;
	}

	public void setId(Serializable id) {
		this.id = id;
	}

	public Serializable getId() {
		return id;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(622, 235);
		this.setLocation(387,0);
		this.setResizable(false);
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.setContentPane(getJContentPane());
		this.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
		{
			public void internalFrameClosing(javax.swing.event.InternalFrameEvent evt)
			{
				clean();
				dispose();
			}
		});
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			logoLabel = new JLabel();
			logoLabel.setBounds(new Rectangle(497, 65, 104, 65));
			logoLabel.setText("");
			pathLabel = new JLabel();
			pathLabel.setBounds(new Rectangle(24, 117, 49, 16));
			pathLabel.setText(Locale.get("BankingWindow.pathLabel"));
			nameLabel = new JLabel();
			nameLabel.setBounds(new Rectangle(24, 72, 49, 16));
			nameLabel.setText(Locale.get("BankingWindow.nameLabel"));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getNameTextField(), null);
			jContentPane.add(getLogoTextField(), null);
			jContentPane.add(getExploreButton(), null);
			jContentPane.add(logoLabel, null);
			jContentPane.add(nameLabel,null);
			jContentPane.add(pathLabel,null);
			jContentPane.add(getSaveButton(), null);
			jContentPane.add(getCancelButton(), null);
			jContentPane.add(getInfoPanel(),null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes nameTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getNameTextField() {
		if (nameTextField == null) {
			nameTextField = new JTextField();
			nameTextField.setBounds(new Rectangle(81, 72, 405, 20));
		}
		return nameTextField;
	}

	/**
	 * This method initializes logoTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getLogoTextField() {
		if (logoTextField == null) {
			logoTextField = new JTextField();
			logoTextField.setBounds(new Rectangle(81, 113, 365, 20));
			logoTextField.setEditable(false);
		}
		return logoTextField;
	}

	/**
	 * This method initializes exploreButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getExploreButton() {
		if (exploreButton == null) {
			exploreButton = new JButton();
			exploreButton.setBounds(new Rectangle(452, 113, 34, 18));
			exploreButton.setText("...");			 
			exploreButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					//Set up the file chooser.
					if (jFileChooser == null) {
						jFileChooser = new JFileChooser();

						//Add a custom file filter and disable the default
						//(Accept All) file filter.
						jFileChooser.addChoosableFileFilter(new ImageFilter());
						jFileChooser.setAcceptAllFileFilterUsed(false);

						//Add custom icons for file types.
						jFileChooser.setFileView(new ImageFileView());

						//Add the preview pane.
						jFileChooser.setAccessory(new ImagePreview(jFileChooser));
					}

					//Show it.
					int returnVal = jFileChooser.showDialog(BankingWindow.this,
							Locale.get("JFileChooser.approveButton"));

					//Process the results.
					if (returnVal == JFileChooser.APPROVE_OPTION) {

						File file = jFileChooser.getSelectedFile();			       

						ImageIcon image = new ImageIcon(file.getPath());

						ImageIcon resizedImage = new ImageIcon(image.getImage().
								getScaledInstance(104, 65,
										Image.SCALE_SMOOTH));

						logoLabel.setIcon(resizedImage);

						logoTextField.setText(file.getPath());

					} else {

					}			      

					//Reset the file chooser for the next time it's shown.
					jFileChooser.setSelectedFile(null);

				}
			});
		}
		return exploreButton;
	}

	/**
	 * This method initializes saveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSaveButton() {
		if (saveButton == null) {
			saveButton = new JButton();
			saveButton.setText(Locale.get("BankingWindow.saveButton"));
			saveButton.setBounds(new Rectangle(411, 151, 91, 20));
			saveButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {					

					HashMap<String, Object> attrs = new HashMap<String, Object>();
					attrs.put("id",getId());
					attrs.put("name", getNameTextField().getText());
					attrs.put("logo", getLogoTextField().getText());

					String message = getDialogMessage();
					String title = getDialogTitle();

					getAdapter().save(attrs);

					if (!inputErrors()) {

						Object[] options = {Locale.get("JOptionPane.Ok")};
						int n = JOptionPane.showOptionDialog(null,
								message,
								title,
								JOptionPane.OK_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);

						if (n == JOptionPane.OK_OPTION || n == JOptionPane.CLOSED_OPTION) {
							ListWindow listWindow = (ListWindow) getOwner();
							listWindow.loadList();
							clean();
							dispose();	                    	
						}					

					}

				}

			});

		}
		return saveButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Locale.get("BankingWindow.cancelButton"));
			cancelButton.setBounds(new Rectangle(510, 151, 91, 20));
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					clean(); 
					dispose();

				}
			});
		}
		return cancelButton;
	}

	private InfoPanel getInfoPanel() {
		if (infoPanel == null) {
			infoPanel = new InfoPanel();
			infoPanel.setBounds(0,0,this.getWidth(),50);
		}
		return infoPanel;
	}

	public void setInfoPanel(InfoPanel ipanel) {		  

		jContentPane.remove(infoPanel);
		infoPanel = ipanel;
		jContentPane.add(infoPanel);

	}

	public void clean() {
		getInfoPanel().setInfoText(getInfoPanel().getInfoText());
		getNameTextField().setText("");
		getLogoTextField().setText("");
		getNameTextField().setBorder(UIManager.getBorder("TextField.border"));
		logoLabel.setIcon(null);
	}		

	public void fill(HashMap fields) {
		getNameTextField().setText((String)fields.get("name"));
		getLogoTextField().setText((String)fields.get("logo"));
		ImageIcon image = new ImageIcon((String)fields.get("logo"));
		logoLabel.setIcon(new ImageIcon(image.getImage().getScaledInstance(104, 65, Image.SCALE_SMOOTH)));
	}

	private boolean inputErrors() {

		if (!ErrorsHandler.getErrors().isEmpty()) {
			
			if (ErrorsHandler.getErrorAt(0).getItem().equals("name")) {

				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("BankingWindow.errors.name")+ "</html>");			
				getNameTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

		}

		return false;

	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
