/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.view.swing.invoicing;

import gz.cyberbrain.cyberhotel.controller.ConceptListAdapter;
import gz.cyberbrain.cyberhotel.controller.InvoiceListAdapter;
import gz.cyberbrain.cyberhotel.controller.ParticularListAdapter;
import gz.cyberbrain.cyberhotel.controller.RoomListAdapter;
import gz.cyberbrain.cyberhotel.controller.SaveAdapter;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.view.swing.ToolsWindow;
import gz.cyberbrain.cyberhotel.view.swing.reception.PayerWindow;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.AutofitTableColumns;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.DateRenderer;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.FloatDocument;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.InfoPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.IntegerDocument;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.NumberEditor;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.OwesList;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.OwesTableModel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.Payable;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.ToolsPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.io.Serializable;
import java.math.BigDecimal;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.border.LineBorder;
import javax.swing.text.Document;
import javax.swing.text.Position;

import org.jdesktop.swingx.JXDatePicker;
import org.jdesktop.swingx.table.DatePickerCellEditor;

public class InvoiceWindow extends ToolsWindow implements Payable, OwesList {

	private static final long serialVersionUID = 1L;
	private static InvoiceWindow invoiceWindow = null;
	private JPanel jContentPane = null;
	private Serializable id = null;
	private Serializable payerId = null;
	private SaveAdapter saveAdapter = null;
	private JInternalFrame owner = null;
	private String dialogTitle = null;
	private String dialogMessage = null;
	private JPanel listPanel = null;
	private InfoPanel infoPanel = null;
	private JLabel findLabel = null;
	private JTextField findTextField = null;
	private JScrollPane listScrollPane = null;
	private JList invoicesList = null;
	private JLabel numLabel = null;
	private JTextField numTextField = null;
	private JLabel serialLabel = null;
	private JTextField serialTextField = null;
	private JLabel dateLabel = null;
	private JXDatePicker dateTextField = null;
	private JLabel shipperCifLabel = null;
	private JTextField shipperCifTextField = null;
	private JLabel shipperNameLabel = null;
	private JTextField shipperNameTextField = null;
	private JLabel shipperStreetLabel = null;
	private JTextField shipperStreetTextField = null;
	private JLabel payerLabel = null;
	private JLabel payModeLabel = null;
	private JComboBox payModeComboBox = null;
	private JLabel subtotalLabel = null;
	private JTextField subtotalTextField = null;
	private JLabel vatLabel = null;
	private JTextField vatTextField = null;
	private JLabel totalLabel = null;
	private JTextField totalTextField = null;
	private JLabel statusLabel = null;
	private JComboBox statusComboBox = null;
	private JLabel observationsLabel = null;
	private JScrollPane observationsScrollPane = null;
	private JTextArea observationsTextArea = null;
	private JScrollPane owesScrollPane = null;
	private JLabel owesLabel = null;
	private JButton saveButton = null;
	private JButton exitButton = null;
	private JTable owesTable = null;
	private JTextField payerTextField = null;
	private JButton payerButton = null;
	private ToolsPanel toolsPanel = null;
	private JButton addButton = null;
	private Object [][]owes = new Object[0][0];
	private int current = 0;
	private boolean callNextOrPrevious = true;

	/**
	 * This is the xxx default constructor
	 */
	public InvoiceWindow() {
		super("", false, true, false, true);
		initialize();
		callNextOrPrevious = true;
	}

	public static InvoiceWindow getInstance() {
		if (invoiceWindow == null) invoiceWindow = new InvoiceWindow();
		return invoiceWindow;
	}

	public void setId(Serializable id) {
		this.id = id;
	}

	public Serializable getId() {
		return id;
	}

	public void setPayerId(Serializable payerId) {
		this.payerId = payerId;
	}

	public Serializable getPayerId() {
		return payerId;
	}

	public void setPayer(Serializable payerId, String payerText) {
		getPayerTextField().setText(payerText);
		setPayerId(payerId);
	}

	public void setAdapter(SaveAdapter saveAdapter) {
		this.saveAdapter = saveAdapter;
	}

	public SaveAdapter getAdapter() {
		return saveAdapter;
	}

	public JInternalFrame getOwner() {
		return owner;
	}

	public void setOwner(JInternalFrame owner) {
		this.owner = owner;
	}

	public String getDialogMessage() {
		return dialogMessage;
	}

	public void setDialogMessage(String dialogMessage) {
		this.dialogMessage = dialogMessage;
	}

	public String getDialogTitle() {
		return dialogTitle;
	}

	public void setDialogTitle(String dialogTitle) {
		this.dialogTitle = dialogTitle;
	}


	/**
	 * This method initializes toolsPanel	
	 * 	
	 * @return gz.cyberbrain.cyberhotel.view.swing.util.gui.ToolsPanel	
	 */
	private ToolsPanel getToolsPanel() {
		if (toolsPanel == null) {
			toolsPanel = new ToolsPanel(this, new InvoiceListAdapter());
			toolsPanel.setBounds(new Rectangle(226, 535, this.getWidth(), 50));
		}
		return toolsPanel;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(866, 625);
		this.setContentPane(getJContentPane());
		this.setResizable(false);
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter() {

			public void internalFrameClosed(javax.swing.event.InternalFrameEvent evt) {

				(InvoiceWindow.getInstance()).clean();
				(InvoiceWindow.getInstance()).lock();				
				getToolsPanel().reset();
				invoiceWindow = null;		

			}

		});
		lock();
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			owesLabel = new JLabel();
			owesLabel.setBounds(new Rectangle(227, 306, 49, 16));
			owesLabel.setText(Locale.get("InvoiceWindow.owesLabel"));
			observationsLabel = new JLabel();
			observationsLabel.setText(Locale.get("InvoiceWindow.observationsLabel"));
			observationsLabel.setBounds(new Rectangle(227, 213, 97, 16));
			statusLabel = new JLabel();
			statusLabel.setBounds(new Rectangle(670, 188, 62, 16));
			statusLabel.setText(Locale.get("InvoiceWindow.statusLabel"));
			totalLabel = new JLabel();
			totalLabel.setBounds(new Rectangle(478, 188, 43, 16));
			totalLabel.setText(Locale.get("InvoiceWindow.totalLabel"));
			vatLabel = new JLabel();
			vatLabel.setBounds(new Rectangle(370, 188, 38, 16));
			vatLabel.setText(Locale.get("InvoiceWindow.vatLabel"));
			subtotalLabel = new JLabel();
			subtotalLabel.setBounds(new Rectangle(228, 188, 49, 16));
			subtotalLabel.setText(Locale.get("InvoiceWindow.subtotalLabel"));
			payModeLabel = new JLabel();
			payModeLabel.setBounds(new Rectangle(670, 160, 62, 16));
			payModeLabel.setText(Locale.get("InvoiceWindow.payModeLabel"));
			payerLabel = new JLabel();
			payerLabel.setBounds(new Rectangle(228, 160, 49, 16));
			payerLabel.setText(Locale.get("InvoiceWindow.payerLabel"));
			shipperStreetLabel = new JLabel();
			shipperStreetLabel.setBounds(new Rectangle(369, 116, 49, 16));
			shipperStreetLabel.setText(Locale.get("InvoiceWindow.shipperStreetLabel"));
			shipperNameLabel = new JLabel();
			shipperNameLabel.setBounds(new Rectangle(369, 86, 49, 16));
			shipperNameLabel.setText(Locale.get("InvoiceWindow.shipperNameLabel"));
			shipperCifLabel = new JLabel();
			shipperCifLabel.setBounds(new Rectangle(227, 88, 49, 16));
			shipperCifLabel.setText(Locale.get("InvoiceWindow.shipperCifLabel"));
			dateLabel = new JLabel();
			dateLabel.setBounds(new Rectangle(721, 61, 38, 16));
			dateLabel.setText(Locale.get("InvoiceWindow.dateLabel"));
			serialLabel = new JLabel();
			serialLabel.setBounds(new Rectangle(369, 61, 49, 16));
			serialLabel.setText(Locale.get("InvoiceWindow.serialLabel"));
			numLabel = new JLabel();
			numLabel.setBounds(new Rectangle(227, 61, 49, 16));
			numLabel.setText(Locale.get("InvoiceWindow.numLabel"));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getListPanel(), null);
			jContentPane.add(getInfoPanel(), null);
			jContentPane.add(numLabel, null);
			jContentPane.add(getNumTextField(), null);
			jContentPane.add(serialLabel, null);
			jContentPane.add(getSerialTextField(), null);
			jContentPane.add(dateLabel, null);
			jContentPane.add(getDateTextField(), null);
			jContentPane.add(shipperCifLabel, null);
			jContentPane.add(getShipperCifTextField(), null);
			jContentPane.add(shipperNameLabel, null);
			jContentPane.add(getShipperNameTextField(), null);
			jContentPane.add(shipperStreetLabel, null);
			jContentPane.add(getShipperStreetTextField(), null);
			jContentPane.add(payerLabel, null);
			jContentPane.add(payModeLabel, null);
			jContentPane.add(getPayModeComboBox(), null);
			jContentPane.add(subtotalLabel, null);
			jContentPane.add(getSubtotalTextField(), null);
			jContentPane.add(vatLabel, null);
			jContentPane.add(getVatTextField(), null);
			jContentPane.add(totalLabel, null);
			jContentPane.add(getTotalTextField(), null);
			jContentPane.add(statusLabel, null);
			jContentPane.add(getStatusComboBox(), null);
			jContentPane.add(getObservationsScrollPane(), null);
			jContentPane.add(getOwesScrollPane(), null);
			jContentPane.add(owesLabel, null);
			jContentPane.add(getSaveButton(), null);
			jContentPane.add(getExitButton(), null);
			jContentPane.add(getPayerTextField(), null);
			jContentPane.add(getPayerButton(), null);
			jContentPane.add(observationsLabel, null);
			jContentPane.add(getToolsPanel(), null);
			jContentPane.add(getAddButton(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes listPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getListPanel() {
		if (listPanel == null) {
			findLabel = new JLabel();
			findLabel.setBounds(new Rectangle(8, 10, 46, 16));
			findLabel.setText(Locale.get("InvoiceWindow.findLabel"));
			listPanel = new JPanel();
			listPanel.setLayout(null);
			listPanel.setBounds(new Rectangle(0, 50, 218, 543));
			listPanel.add(findLabel, null);
			listPanel.add(getFindTextField(), null);
			listPanel.add(getListScrollPane(), null);
		}
		return listPanel;
	}

	/**
	 * This method initializes infoPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private InfoPanel getInfoPanel() {
		if (infoPanel == null) {
			infoPanel = new InfoPanel();
			infoPanel.setBounds(new Rectangle(0, 0, this.getWidth(), 50));
		}
		return infoPanel;
	}

	public void setInfoPanel(InfoPanel ipanel) {		  

		jContentPane.remove(infoPanel);
		infoPanel = ipanel;
		jContentPane.add(infoPanel);

	}

	/**
	 * This method initializes findTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getFindTextField() {
		if (findTextField == null) {
			findTextField = new JTextField();
			findTextField.setBounds(new Rectangle(72, 9, 138, 20));
			findTextField.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyPressed(java.awt.event.KeyEvent e) {

					String text = findTextField.getText(); 

					try {

						if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE && text.length() != 0) getInvoicesList().setSelectedIndex(getInvoicesList().getNextMatch(text.substring(0,text.length() - 1), 0, Position.Bias.Forward));
						else getInvoicesList().setSelectedIndex(getInvoicesList().getNextMatch(text + e.getKeyChar(), 0, Position.Bias.Forward));

					} catch (IllegalArgumentException f) {} 

				}
			});
		}
		return findTextField;
	}

	/**
	 * This method initializes listScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getListScrollPane() {
		if (listScrollPane == null) {
			listScrollPane = new JScrollPane();
			listScrollPane.setBounds(new Rectangle(8, 34, 202, 500));
			listScrollPane.setViewportView(getInvoicesList());			
		}
		return listScrollPane;
	}

	private void loadList() {

		DefaultListModel model = new DefaultListModel();	

		Vector data = (new InvoiceListAdapter()).load();

		for (int i=0; i < data.size(); i++) {
			model.add(i, data.elementAt(i));
		}

		invoicesList.setModel(model);

		/*	if (invoicesList.getComponentCount() != 0) {

			invoicesList.setSelectedIndex(0);

			fill((new InvoiceListAdapter()).find(((TextItem) invoicesList.getSelectedValue()).getId()));

			setId(((TextItem) invoicesList.getSelectedValue()).getId());

		}*/

	}

	/**
	 * This method initializes invoicesList	
	 * 	
	 * @return javax.swing.JList	
	 */
	private JList getInvoicesList() {
		if (invoicesList == null) {
			invoicesList = new JList();
			loadList();
			invoicesList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
			{
				public void valueChanged(javax.swing.event.ListSelectionEvent evt)
				{

					if(callNextOrPrevious) {

						for (int i=0; i < java.lang.Math.abs(invoicesList.getSelectedIndex() - current); i++) {
							if (current < invoicesList.getSelectedIndex()) getToolsPanel().next();
							else getToolsPanel().previous();
						}
					}
					current = invoicesList.getSelectedIndex();

					//If we reload the list
					if (current == -1) {
						invoicesList.setSelectedIndex(0);
						current = 0;
					}

					owes = null;
					owes = new Object[0][0];
					fill((new InvoiceListAdapter()).find(((TextItem) invoicesList.getSelectedValue()).getId()));
					setId(((TextItem) invoicesList.getSelectedValue()).getId());

					callNextOrPrevious = true;
				}
			});
		}
		return invoicesList;
	}

	/**
	 * This method initializes numTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getNumTextField() {
		if (numTextField == null) {
			Document integerDocument = new IntegerDocument(16);
			numTextField = new JTextField(integerDocument,"",16);
			numTextField.setBounds(new Rectangle(281, 59, 82, 20));
		}
		return numTextField;
	}

	/**
	 * This method initializes serialTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getSerialTextField() {
		if (serialTextField == null) {
			serialTextField = new JTextField();
			serialTextField.setBounds(new Rectangle(424, 59, 39, 20));
		}
		return serialTextField;
	}

	/**
	 * This method initializes dateTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JXDatePicker getDateTextField() {
		if (dateTextField == null) {
			dateTextField = new JXDatePicker();
			dateTextField.setBounds(new Rectangle(763, 59, 84, 20));
			DateFormat []df = {DateFormat.getDateInstance(DateFormat.SHORT)}; 
			dateTextField.setFormats(df);
		}
		return dateTextField;
	}

	/**
	 * This method initializes shipperCifTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getShipperCifTextField() {
		if (shipperCifTextField == null) {
			shipperCifTextField = new JTextField();
			shipperCifTextField.setBounds(new Rectangle(281, 87, 82, 20));
		}
		return shipperCifTextField;
	}

	/**
	 * This method initializes shipperNameTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getShipperNameTextField() {
		if (shipperNameTextField == null) {
			shipperNameTextField = new JTextField();
			shipperNameTextField.setBounds(new Rectangle(424, 87, 422, 20));
		}
		return shipperNameTextField;
	}

	/**
	 * This method initializes shipperStreetTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getShipperStreetTextField() {
		if (shipperStreetTextField == null) {
			shipperStreetTextField = new JTextField();
			shipperStreetTextField.setBounds(new Rectangle(424, 115, 422, 20));
		}
		return shipperStreetTextField;
	}

	/**
	 * This method initializes payModeComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getPayModeComboBox() {
		if (payModeComboBox == null) {
			payModeComboBox = new JComboBox();
			payModeComboBox.setBounds(new Rectangle(739, 159, 108, 19));
			payModeComboBox.addItem(Locale.get("InvoiceWindow.payModeComboBox.item0"));
			payModeComboBox.addItem(Locale.get("InvoiceWindow.payModeComboBox.item1"));
			payModeComboBox.addItem(Locale.get("InvoiceWindow.payModeComboBox.item2"));
		}
		return payModeComboBox;
	}

	/**
	 * This method initializes subtotalTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getSubtotalTextField() {
		if (subtotalTextField == null) {
			Document floatDocument = new FloatDocument();
			subtotalTextField = new JTextField(floatDocument,"",16);
			subtotalTextField.setBounds(new Rectangle(282, 185, 81, 20));
			subtotalTextField.setText("0");
			subtotalTextField.addFocusListener( new FocusListener() {

				public void focusLost(FocusEvent e) {
					if (subtotalTextField.getText().trim().equals("")) subtotalTextField.setText("0");
				}

				public void focusGained(FocusEvent arg0) {}

			});
		}

		return subtotalTextField;
	}

	/**
	 * This method initializes vatTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getVatTextField() {
		if (vatTextField == null) {
			Document floatDocument = new FloatDocument();
			vatTextField = new JTextField(floatDocument,"",16);
			vatTextField.setBounds(new Rectangle(425, 185, 50, 20));
			vatTextField.setText("0");
			vatTextField.addFocusListener( new FocusListener() {

				public void focusLost(FocusEvent e) {
					if (vatTextField.getText().trim().equals("")) vatTextField.setText("0");
				}

				public void focusGained(FocusEvent arg0) {}

			});
		}
		return vatTextField;
	}

	/**
	 * This method initializes totalTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getTotalTextField() {
		if (totalTextField == null) {
			Document floatDocument = new FloatDocument();
			totalTextField = new JTextField(floatDocument,"",16);
			totalTextField.setBounds(new Rectangle(530, 185, 85, 20));
			totalTextField.setText("0");
			totalTextField.addFocusListener( new FocusListener() {

				public void focusLost(FocusEvent e) {
					if (totalTextField.getText().trim().equals("")) totalTextField.setText("0");
				}

				public void focusGained(FocusEvent arg0) {}

			});
		}
		return totalTextField;
	}

	/**
	 * This method initializes statusComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getStatusComboBox() {
		if (statusComboBox == null) {
			statusComboBox = new JComboBox();
			statusComboBox.setBounds(new Rectangle(739, 185, 108, 19));
			statusComboBox.addItem(new TextItem("Pendente", Locale.get("InvoiceWindow.statusComboBox.outstanding")));
			statusComboBox.addItem(new TextItem("Emitida", Locale.get("InvoiceWindow.statusComboBox.issued")));
		}
		return statusComboBox;
	}

	/**
	 * This method initializes observationsScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getObservationsScrollPane() {
		if (observationsScrollPane == null) {
			observationsScrollPane = new JScrollPane();
			observationsScrollPane.setBounds(new Rectangle(226, 238, 623, 57));
			observationsScrollPane.setViewportView(getObservationsTextArea());
		}
		return observationsScrollPane;
	}

	/**
	 * This method initializes observationsTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getObservationsTextArea() {
		if (observationsTextArea == null) {
			observationsTextArea = new JTextArea();
		}
		return observationsTextArea;
	}

	/**
	 * This method initializes owesScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getOwesScrollPane() {
		if (owesScrollPane == null) {
			owesScrollPane = new JScrollPane();
			owesScrollPane.setBounds(new Rectangle(226, 330, 623, 200));
			owesScrollPane.setViewportView(getOwesTable());
		}
		return owesScrollPane;
	}

	/**
	 * This method initializes saveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSaveButton() {
		if (saveButton == null) {
			saveButton = new JButton();
			saveButton.setBounds(new Rectangle(693, 552, 71, 16));
			saveButton.setText(Locale.get("InvoiceWindow.saveButton"));
			saveButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					HashMap<String, Serializable> attrs = new HashMap<String, Serializable>();					

					attrs.put("id", (String) getId());
					attrs.put("invoiceNumber", getNumTextField().getText());					           
					attrs.put("invoiceSerial", getSerialTextField().getText());
					Calendar date = Calendar.getInstance();
					date.setTime(getDateTextField().getDate());
					attrs.put("invoiceDate", date);
					attrs.put("shipperName",getShipperNameTextField().getText());
					attrs.put("shipperCif", getShipperCifTextField().getText());
					attrs.put("shipperStreet", getShipperStreetTextField().getText());
					attrs.put("payer", (InvoiceWindow.getInstance()).getPayerId());

					String payMode = null;
					switch (getPayModeComboBox().getSelectedIndex()) {
					case 0:
						payMode = "Contado";
						break;
					case 1:
						payMode = "Tarxeta";
						break;
					case 2:
						payMode = "Bono";
						break;
					}

					attrs.put("payMode", payMode); 

					attrs.put("subtotal", getSubtotalTextField().getText().replace(',', '.'));
					attrs.put("quotaVat", getVatTextField().getText().replace(',', '.'));
					attrs.put("total", getTotalTextField().getText().replace(',', '.'));		

					String status = null;
					switch (getStatusComboBox().getSelectedIndex()) {
					case 0:
						status = "Pendente";
						break;
					case 1:
						status = "Emitida";
						break;
					}

					attrs.put("status", status);

					attrs.put("observations", getObservationsTextArea().getText());

					Vector <Long> oweNumbers = new Vector <Long> ();

					for (int i=0; i<getOwesTable().getRowCount(); i++) {

						oweNumbers.add((Long) ((OwesTableModel) getOwesTable().getModel()).getCustomValueAt(i, 8));

					}

					setId(getAdapter().save(attrs, oweNumbers));

					if (!inputErrors()) {										

						setDialogMessage(Locale.get("JOptionPane.InvoiceWindow.saveButton.message"));
						setDialogTitle(Locale.get("JOptionPane.InvoiceWindow.saveButton.title"));

						Object[] options = {Locale.get("JOptionPane.Ok")};
						int n = JOptionPane.showOptionDialog(InvoiceWindow.getInstance(),
								getDialogMessage(),
								getDialogTitle(),
								JOptionPane.OK_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);

						if (n == JOptionPane.OK_OPTION) {							
							clean();
							update();
							lock();
							getToolsPanel().reset();					
						}																	

					}					 					 

				}

			});
		}
		return saveButton;
	}

	/**
	 * This method initializes exitButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getExitButton() {
		if (exitButton == null) {
			exitButton = new JButton();
			exitButton.setBounds(new Rectangle(778, 552, 71, 16));
			exitButton.setText(Locale.get("InvoiceWindow.exitButton"));
			exitButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					(InvoiceWindow.getInstance()).dispose();
					invoiceWindow = null;

				}
			});
		}
		return exitButton;
	}

	private Object [][] loadData() {

		return owes;

	}

	private String[] loadTitles() {

		String [] titles = {
				Locale.get("InvoiceWindow.owesTable.firstColumn.title"),
				Locale.get("InvoiceWindow.owesTable.secondColumn.title"),
				Locale.get("InvoiceWindow.owesTable.thirdColumn.title"),
				Locale.get("InvoiceWindow.owesTable.fourthColumn.title"),
				Locale.get("InvoiceWindow.owesTable.fifthColumn.title"),
				Locale.get("InvoiceWindow.owesTable.sixthColumn.title"),
				Locale.get("InvoiceWindow.owesTable.seventhColumn.title"),
				Locale.get("InvoiceWindow.owesTable.eighthColumn.title")
		};

		return titles;
	}

	@SuppressWarnings("unchecked")
	private Vector getConcepts() {
		Vector concepts = (new ConceptListAdapter()).load();
		concepts.insertElementAt(new TextItem(new String(""),""), 0);
		return concepts;
	}

	@SuppressWarnings("unchecked")
	private Vector getRooms() {
		Vector rooms = (new RoomListAdapter()).load();
		rooms.insertElementAt(new TextItem(new String(""),""), 0);
		return rooms;
	}

	@SuppressWarnings("unchecked")
	private Vector getParticulars() {
		Vector particulars = (new ParticularListAdapter()).load();
		particulars.insertElementAt(new TextItem(new Long(0),""), 0);
		return particulars;
	}

	private Vector getOwesStatus() {

		Vector <TextItem> status = new Vector<TextItem>();
		status.add(new TextItem("Pendente", Locale.get("OwesDialog.statusComboBox.outstanding")));
		status.add(new TextItem("Facturado", Locale.get("OwesDialog.statusComboBox.billed")));
		return status;

	}

	/**
	 * This method initializes owesTable	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getOwesTable() {
		if (owesTable == null) {
			owesTable = new JTable() {

				private static final long serialVersionUID = 1L;

				public boolean isCellEditable(int row, int col) {
					return false;
				}								

			};

			owesTable.setModel(new OwesTableModel(loadData(), loadTitles()));
			//if (owesTable.getRowCount() > 0 ) owesTable.setRowSelectionInterval(0, 0);
			owesTable.getColumnModel().getColumn(0).setCellEditor(new DatePickerCellEditor(DateFormat.getDateInstance(DateFormat.SHORT)));
			owesTable.getColumnModel().getColumn(0).setCellRenderer(new DateRenderer());			
			owesTable.getColumnModel().getColumn(1).setCellEditor(new DefaultCellEditor(new JComboBox(getRooms())));
			owesTable.getColumnModel().getColumn(2).setCellEditor(new DefaultCellEditor(new JComboBox(getParticulars())));			
			owesTable.getColumnModel().getColumn(3).setCellEditor(new DefaultCellEditor(new JComboBox(getConcepts())));
			owesTable.getColumnModel().getColumn(4).setCellEditor(new NumberEditor());
			owesTable.getColumnModel().getColumn(5).setCellEditor(new NumberEditor());
			owesTable.getColumnModel().getColumn(6).setCellEditor(new NumberEditor());
			owesTable.getColumnModel().getColumn(7).setCellEditor(new DefaultCellEditor(new JComboBox(getOwesStatus())));

			owesTable.getModel().addTableModelListener(new javax.swing.event.TableModelListener() {

				public void tableChanged(javax.swing.event.TableModelEvent e) {}

			});

			AutofitTableColumns.autoResizeTable(owesTable, true);

		}
		return owesTable;
	}

	/**
	 * This method initializes payerTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getPayerTextField() {
		if (payerTextField == null) {
			payerTextField = new JTextField();
			payerTextField.setBounds(new Rectangle(282, 160, 333, 20));
			payerTextField.setEditable(false);
		}
		return payerTextField;
	}

	/**
	 * This method initializes payerButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getPayerButton() {
		if (payerButton == null) {
			payerButton = new JButton();
			payerButton.setBounds(new Rectangle(619, 160, 34, 20));
			payerButton.setText(Locale.get("InvoiceWindow.payerButton"));
			payerButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					PayerWindow payerWindow = new PayerWindow();
					payerWindow.setObserver(InvoiceWindow.getInstance());
					payerWindow.setVisible(true);
				}
			});
		}
		return payerButton;
	}

	@Override
	public void cleanErrors() {

		getInfoPanel().setInfoText(getInfoPanel().getInfoText());

		getNumTextField().setBorder(UIManager.getBorder("TextField.border"));
		getSerialTextField().setBorder(UIManager.getBorder("TextField.border"));
		getPayerTextField().setBorder(UIManager.getBorder("TextField.border"));
		getOwesTable().setBorder(UIManager.getBorder("Table.border"));		

	}

	@Override
	public void lock() {

		getNumTextField().setEditable(false);
		getSerialTextField().setEditable(false);
		getDateTextField().setEditable(false);
		getShipperCifTextField().setEditable(false);
		getShipperNameTextField().setEditable(false);
		getShipperStreetTextField().setEditable(false);
		getPayerButton().setEnabled(false);
		getPayModeComboBox().setEnabled(false);
		getSubtotalTextField().setEditable(false);
		getVatTextField().setEditable(false);
		getTotalTextField().setEditable(false);
		getStatusComboBox().setEnabled(false);
		getObservationsTextArea().setEditable(false);
		getAddButton().setEnabled(false);
		getSaveButton().setEnabled(false);

	}

	@Override
	public boolean showErrors() {
		return inputErrors();
	}

	@Override
	public void unlock() {

		//getFindTextField().setEditable(true);
		//getInvoicesList().setEnabled(true);
		getNumTextField().setEditable(true);
		getSerialTextField().setEditable(true);
		getDateTextField().setEditable(true);
		getShipperCifTextField().setEditable(true);
		getShipperNameTextField().setEditable(true);
		getShipperStreetTextField().setEditable(true);
		getPayerButton().setEnabled(true);
		getPayModeComboBox().setEnabled(true);
		getSubtotalTextField().setEditable(true);
		getVatTextField().setEditable(true);
		getTotalTextField().setEditable(true);
		getStatusComboBox().setEnabled(true);
		getObservationsTextArea().setEditable(true);
		getAddButton().setEnabled(true);
		getSaveButton().setEnabled(true);

	}

	@Override
	public void update() {
		loadList();
	}

	@Override
	public void clean() {

		//getFindTextField().setEditable(false);
		//getInvoicesList().setEnabled(false);
		getNumTextField().setText("");
		getSerialTextField().setText("");
		getDateTextField().setDate((Calendar.getInstance()).getTime());
		getShipperCifTextField().setText("");
		getShipperNameTextField().setText("");
		getShipperStreetTextField().setText("");
		getPayerTextField().setText("");
		setPayer(null, null);
		getPayModeComboBox().setSelectedIndex(0);
		getStatusComboBox().setSelectedIndex(0);
		getSubtotalTextField().setText("0");
		getVatTextField().setText("0");
		getTotalTextField().setText("0");
		getObservationsTextArea().setText("");
		owes = new Object[0][0];
		refreshOwesTable();
		
	}

	@Override
	public void fill(HashMap fields) {

		setId((String) fields.get("id"));

		callNextOrPrevious = false;
		getInvoicesList().setSelectedIndex(getInvoicesList().getNextMatch(((String) getId()), 0, Position.Bias.Forward));
		setOwesList((Object[][]) fields.get("invoiceOwes"));
		refreshOwesTable();

		getNumTextField().setText((String) ((Long) fields.get("invoiceNumber")).toString());
		getSerialTextField().setText((String) fields.get("invoiceSerial"));
		getDateTextField().setDate(((Calendar) fields.get("invoiceDate")).getTime()); 
		getShipperNameTextField().setText((String) fields.get("shipperName"));
		getShipperCifTextField().setText((String) fields.get("shipperCif"));
		getShipperStreetTextField().setText((String) fields.get("shipperStreet"));

		getPayerTextField().setText(((TextItem) fields.get("payer")).getText());
		setPayer(((TextItem) fields.get("payer")).getId(),((TextItem) fields.get("payer")).getText());

		if(((String) fields.get("payMode")).equals("Contado")) getPayModeComboBox().setSelectedIndex(0);
		if(((String) fields.get("payMode")).equals("Tarxeta")) getPayModeComboBox().setSelectedIndex(1);
		if(((String) fields.get("payMode")).equals("Bono")) getPayModeComboBox().setSelectedIndex(2);

		getSubtotalTextField().setText(((BigDecimal) fields.get("subtotal")).toString());
		getVatTextField().setText(((BigDecimal) fields.get("quotaVat")).toString());
		getTotalTextField().setText(((BigDecimal) fields.get("total")).toString());		

		if (((String) fields.get("status")).equals("Pendente")) getStatusComboBox().setSelectedIndex(0);
		if (((String) fields.get("status")).equals("Emitida")) getStatusComboBox().setSelectedIndex(1);

		getObservationsTextArea().setText((String) fields.get("observations"));

	}

	/**
	 * This method initializes addButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton();
			addButton.setBounds(new Rectangle(766, 306, 81, 16));
			addButton.setText(Locale.get("InvoiceWindow.addButton"));
			addButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					OwesDialog owesDialog = new OwesDialog();
					owesDialog.setObserver(InvoiceWindow.getInstance());
					owesDialog.setVisible(true);
				}
			});

		}
		return addButton;
	}

	private void refreshOwesTable() {
		owesTable.setModel(new OwesTableModel(loadData(), loadTitles()));
		//if (owesTable.getRowCount() > 0 ) owesTable.setRowSelectionInterval(0, 0);
		owesTable.getColumnModel().getColumn(0).setCellEditor(new DatePickerCellEditor(DateFormat.getDateInstance(DateFormat.SHORT)));
		owesTable.getColumnModel().getColumn(0).setCellRenderer(new DateRenderer());			
		owesTable.getColumnModel().getColumn(1).setCellEditor(new DefaultCellEditor(new JComboBox(getRooms())));
		owesTable.getColumnModel().getColumn(2).setCellEditor(new DefaultCellEditor(new JComboBox(getParticulars())));			
		owesTable.getColumnModel().getColumn(3).setCellEditor(new DefaultCellEditor(new JComboBox(getConcepts())));
		owesTable.getColumnModel().getColumn(4).setCellEditor(new NumberEditor());
		owesTable.getColumnModel().getColumn(5).setCellEditor(new NumberEditor());
		owesTable.getColumnModel().getColumn(6).setCellEditor(new NumberEditor());
		owesTable.getColumnModel().getColumn(7).setCellEditor(new DefaultCellEditor(new JComboBox(getOwesStatus())));

		owesTable.getModel().addTableModelListener(new javax.swing.event.TableModelListener() {

			public void tableChanged(javax.swing.event.TableModelEvent e) {}

		});

		AutofitTableColumns.autoResizeTable(owesTable, true);

		owesTable.repaint();

	}	

	public void setOwesList(Object[][] owes) {

		int newLength = 0;
		
		for(int i=0; i<owes.length; i++) 
			if (!repeatedOwe((Long) owes[i][8])) newLength++;
		
		Object [][]newOwes = new Object[this.owes.length + newLength][getOwesTable().getColumnCount() + 1];

		int oldSize = this.owes.length;

		//Insert old rows if new owes exist

		for(int i=0; i<oldSize; i++) 
			for (int j=0; j<getOwesTable().getColumnCount() + 1; j++) newOwes[i][j] = this.owes[i][j];				
		

		int i=0;
		int k=0;

		//Append new rows if not exists
		while(i<owes.length) {

			if (!repeatedOwe((Long) owes[i][8])) {

				for (int j=0; j<getOwesTable().getColumnCount() + 1; j++) 
					newOwes[oldSize + k][j] = owes[i][j];
				k++;
			}

			i++;
		}

		this.owes = null;
		this.owes = newOwes;

		refreshOwesTable();
	}

	private boolean repeatedOwe(Long id) {

		for(int i=0; i<this.owes.length; i++)
			if (this.owes[i][8] == id) return true;

		return false;

	}

	private boolean inputErrors() {

		if (!ErrorsHandler.getErrors().isEmpty())  {

			if (ErrorsHandler.getErrorAt(0).getItem().equals("invoiceNumber")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("InvoiceWindow.errors.invoiceNumber")+ "</html>");

				getNumTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("invoiceSerial")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("InvoiceWindow.errors.invoiceSerial")+ "</html>");

				getSerialTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("unaceptablePayer")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("InvoiceWindow.errors.unaceptablePayer")+ "</html>");

				getPayerTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("inexistentOwes")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("InvoiceWindow.errors.inexistentOwes")+ "</html>");

				getOwesTable().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("billedOwes")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("InvoiceWindow.errors.billedOwes")+ "</html>");

				getOwesTable().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}	

			if (ErrorsHandler.getErrorAt(0).getItem().equals("payedInvoice")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("InvoiceWindow.errors.payedInvoice")+ "</html>");

				Toolkit.getDefaultToolkit().beep();
				return true;

			}	
			
			
		}

		return false;

	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
