/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.view.swing.invoicing;

import gz.cyberbrain.cyberhotel.controller.OutstandingInvoiceListAdapter;
import gz.cyberbrain.cyberhotel.controller.PaymentListAdapter;
import gz.cyberbrain.cyberhotel.controller.SaveAdapter;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.security.SecurityHandler;
import gz.cyberbrain.cyberhotel.view.swing.ToolsWindow;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.FloatDocument;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.InfoPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.ToolsPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.io.Serializable;
import java.math.BigDecimal;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.border.LineBorder;
import javax.swing.text.Document;
import javax.swing.text.Position;

import org.jdesktop.swingx.JXDatePicker;

public class PaymentWindow extends ToolsWindow {

	private static final long serialVersionUID = 1L;
	private static PaymentWindow paymentWindow = null;
	private Serializable id = null;
	private SaveAdapter saveAdapter = null;
	private String dialogMessage = null;
	private String dialogTitle = null;
	private JPanel jContentPane = null;
	private InfoPanel infoPanel = null;
	private JPanel firstPanel = null;
	private JLabel searchLabel = null;
	private JTextField searchTextField = null;
	private JScrollPane listScrollPane = null;
	private JList invoicesList = null;
	private JLabel referenceLabel = null;
	private JTextField referenceTextField = null;
	private JLabel dateLabel = null;
	private JXDatePicker dateTextField = null;
	private JLabel invoiceNumberLabel = null;
	private JTextField invoiceNumberTextField = null;
	private JLabel invoiceSerialLabel = null;
	private JTextField invoiceSerialTextField = null;
	private JLabel amountLabel = null;
	private JTextField amountTextField = null;
	private JLabel payModeLabel = null;
	private JComboBox payModeComboBox = null;
	private JLabel statusLabel = null;
	private JComboBox statusComboBox = null;
	private JLabel notesLabel = null;
	private JScrollPane notesScrollPane = null;
	private JTextArea notesTextArea = null;
	private JButton saveButton = null;
	private JButton cancelButton = null;
	private ToolsPanel toolsPanel = null;
	private JButton payButton = null;

	/**
	 * This is the xxx default constructor
	 */
	private PaymentWindow() {
		super("", false, true, false, true);
		initialize();
	}

	public static PaymentWindow getInstance() {
		if (paymentWindow == null) paymentWindow = new PaymentWindow();
		return paymentWindow;
	}

	public void setAdapter(SaveAdapter saveAdapter) {
		this.saveAdapter = saveAdapter;
	}

	public SaveAdapter getAdapter() {
		return saveAdapter;
	}

	public void setId(Serializable id) {
		this.id = id;
	}

	public Serializable getId() {
		return id;
	}

	public String getDialogMessage() {
		return dialogMessage;
	}

	public void setDialogMessage(String dialogMessage) {
		this.dialogMessage = dialogMessage;
	}

	public String getDialogTitle() {
		return dialogTitle;
	}

	public void setDialogTitle(String dialogTitle) {
		this.dialogTitle = dialogTitle;
	}

	/**
	 * This method initializes infoPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private InfoPanel getInfoPanel() {
		if (infoPanel == null) {
			infoPanel = new InfoPanel();
			infoPanel.setBounds(new Rectangle(0, 0, this.getWidth(), 50));
		}
		return infoPanel;
	}

	public void setInfoPanel(InfoPanel ipanel) {		  

		infoPanel = ipanel;
		jContentPane.add(infoPanel);

	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(638, 495);
		this.setContentPane(getJContentPane());
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
		{

			public void internalFrameClosed(javax.swing.event.InternalFrameEvent evt)
			{
				(PaymentWindow.getInstance()).clean();
				(PaymentWindow.getInstance()).lock();
				getToolsPanel().reset();
				paymentWindow = null;
			}

		});
		lock();
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			notesLabel = new JLabel();
			notesLabel.setBounds(new Rectangle(224, 266, 38, 16));
			notesLabel.setText(Locale.get("PaymentWindow.notesLabel"));
			statusLabel = new JLabel();
			statusLabel.setBounds(new Rectangle(464, 86, 46, 16));
			statusLabel.setText(Locale.get("PaymentWindow.statusLabel"));
			payModeLabel = new JLabel();
			payModeLabel.setBounds(new Rectangle(464, 160, 45, 16));
			payModeLabel.setText(Locale.get("PaymentWindow.payModeLabel"));
			amountLabel = new JLabel();
			amountLabel.setBounds(new Rectangle(223, 188, 54, 16));
			amountLabel.setText(Locale.get("PaymentWindow.amountLabel"));
			invoiceSerialLabel = new JLabel();
			invoiceSerialLabel.setBounds(new Rectangle(383, 160, 35, 16));
			invoiceSerialLabel.setText(Locale.get("PaymentWindow.invoiceSerialLabel"));
			invoiceNumberLabel = new JLabel();
			invoiceNumberLabel.setBounds(new Rectangle(223, 160, 45, 16));
			invoiceNumberLabel.setText(Locale.get("PaymentWindow.invoiceNumberLabel"));
			dateLabel = new JLabel();
			dateLabel.setBounds(new Rectangle(464, 61, 46, 16));
			dateLabel.setText(Locale.get("PaymentWindow.dateLabel"));
			referenceLabel = new JLabel();
			referenceLabel.setBounds(new Rectangle(224, 63, 38, 16));
			referenceLabel.setText(Locale.get("PaymentWindow.referenceLabel"));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getFirstPanel(), null);
			jContentPane.add(referenceLabel, null);
			jContentPane.add(getReferenceTextField(), null);
			jContentPane.add(dateLabel, null);
			jContentPane.add(getDateTextField(), null);
			jContentPane.add(invoiceNumberLabel, null);
			jContentPane.add(getInvoiceNumberTextField(), null);
			jContentPane.add(invoiceSerialLabel, null);
			jContentPane.add(getInvoiceSerialTextField(), null);
			jContentPane.add(amountLabel, null);
			jContentPane.add(getAmountTextField(), null);
			jContentPane.add(payModeLabel, null);
			jContentPane.add(getPayModeComboBox(), null);
			jContentPane.add(statusLabel, null);
			jContentPane.add(getStatusComboBox(), null);
			jContentPane.add(notesLabel, null);
			jContentPane.add(getNotesScrollPane(), null);
			jContentPane.add(getSaveButton(), null);
			jContentPane.add(getCancelButton(), null);
			jContentPane.add(getToolsPanel(), null);
			jContentPane.add(getInfoPanel(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes firstPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getFirstPanel() {
		if (firstPanel == null) {
			searchLabel = new JLabel();
			searchLabel.setBounds(new Rectangle(7, 11, 38, 16));
			searchLabel.setText(Locale.get("PaymentWindow.searchLabel"));
			firstPanel = new JPanel();
			firstPanel.setLayout(null);
			firstPanel.setBounds(new Rectangle(0, 51, 220, 410));
			firstPanel.add(searchLabel, null);
			firstPanel.add(getSearchTextField(), null);
			firstPanel.add(getListScrollPane(), null);
			firstPanel.add(getPayButton(), null);
		}
		return firstPanel;
	}

	/**
	 * This method initializes searchTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getSearchTextField() {
		if (searchTextField == null) {
			searchTextField = new JTextField();
			searchTextField.setBounds(new Rectangle(55, 11, 157, 20));
			searchTextField.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyPressed(java.awt.event.KeyEvent e) {

					String text = searchTextField.getText(); 

					try {

						if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE && text.length() != 0) getInvoicesList().setSelectedIndex(getInvoicesList().getNextMatch(text.substring(0,text.length() - 1), 0, Position.Bias.Forward));
						else getInvoicesList().setSelectedIndex(getInvoicesList().getNextMatch(text + e.getKeyChar(), 0, Position.Bias.Forward));

					} catch (IllegalArgumentException f) {} 

				}
			});
		}
		return searchTextField;
	}

	/**
	 * This method initializes listScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getListScrollPane() {
		if (listScrollPane == null) {
			listScrollPane = new JScrollPane();
			listScrollPane.setBounds(new Rectangle(5, 36, 207, 279));
			listScrollPane.setViewportView(getInvoicesList());
		}
		return listScrollPane;
	}

	private void loadList() {
		DefaultListModel model = new DefaultListModel();						
		Vector data = (new OutstandingInvoiceListAdapter()).load();
		for (int i=0; i < data.size(); i++) {
			model.add(i, data.elementAt(i));
		}
		invoicesList.setModel(model);
		if (invoicesList.getModel().getSize()!= 0) invoicesList.setSelectedIndex(0);

	}

	/**
	 * This method initializes invoicesList	
	 * 	
	 * @return javax.swing.JList	
	 */
	private JList getInvoicesList() {
		if (invoicesList == null) {
			invoicesList = new JList();
			invoicesList.setEnabled(false);
			loadList();
		}
		return invoicesList;
	}

	/**
	 * This method initializes referenceTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getReferenceTextField() {
		if (referenceTextField == null) {
			referenceTextField = new JTextField();
			referenceTextField.setBounds(new Rectangle(284, 61, 94, 20));
			referenceTextField.setEnabled(false);
		}
		return referenceTextField;
	}

	/**
	 * This method initializes dateTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JXDatePicker getDateTextField() {
		if (dateTextField == null) {
			dateTextField = new JXDatePicker();
			dateTextField.setBounds(new Rectangle(513, 59, 94, 20));
			DateFormat []df = {DateFormat.getDateInstance(DateFormat.SHORT)}; 
			dateTextField.setFormats(df);
		}
		return dateTextField;
	}

	/**
	 * This method initializes invoiceNumberTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getInvoiceNumberTextField() {
		if (invoiceNumberTextField == null) {
			invoiceNumberTextField = new JTextField();
			invoiceNumberTextField.setBounds(new Rectangle(283, 158, 94, 20));
			invoiceNumberTextField.setEnabled(false);
		}
		return invoiceNumberTextField;
	}

	/**
	 * This method initializes invoiceSerialTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getInvoiceSerialTextField() {
		if (invoiceSerialTextField == null) {
			invoiceSerialTextField = new JTextField();
			invoiceSerialTextField.setBounds(new Rectangle(420, 158, 40, 20));
			invoiceSerialTextField.setEnabled(false);
		}
		return invoiceSerialTextField;
	}

	/**
	 * This method initializes amountTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getAmountTextField() {
		if (amountTextField == null) {
			Document floatDocument = new FloatDocument();
			amountTextField = new JTextField(floatDocument, "", 16);
			amountTextField.setBounds(new Rectangle(283, 187, 94, 20));
			amountTextField.setText("0");
			amountTextField.addFocusListener( new FocusListener() {

				public void focusLost(FocusEvent e) {
					if (amountTextField.getText().trim().equals("")) amountTextField.setText("0");
				}

				public void focusGained(FocusEvent arg0) {}

			});
		}
		return amountTextField;
	}

	/**
	 * This method initializes payModeComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getPayModeComboBox() {
		if (payModeComboBox == null) {
			payModeComboBox = new JComboBox();
			payModeComboBox.setBounds(new Rectangle(513, 159, 94, 19));
			payModeComboBox.addItem(Locale.get("PaymentWindow.payModeComboBox.item0"));
			payModeComboBox.addItem(Locale.get("PaymentWindow.payModeComboBox.item1"));
			payModeComboBox.addItem(Locale.get("PaymentWindow.payModeComboBox.item2"));
		}
		return payModeComboBox;
	}

	/**
	 * This method initializes statusComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getStatusComboBox() {
		if (statusComboBox == null) {
			statusComboBox = new JComboBox();
			statusComboBox.setBounds(new Rectangle(513, 86, 94, 19));
			statusComboBox.addItem(Locale.get("PaymentWindow.statusComboBox.item0"));
			statusComboBox.addItem(Locale.get("PaymentWindow.statusComboBox.item1"));
			statusComboBox.setEnabled(false);

		}
		return statusComboBox;
	}

	/**
	 * This method initializes notesScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getNotesScrollPane() {
		if (notesScrollPane == null) {
			notesScrollPane = new JScrollPane();
			notesScrollPane.setBounds(new Rectangle(225, 288, 392, 76));
			notesScrollPane.setViewportView(getNotesTextArea());
		}
		return notesScrollPane;
	}

	/**
	 * This method initializes notesTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getNotesTextArea() {
		if (notesTextArea == null) {
			notesTextArea = new JTextArea();
		}
		return notesTextArea;
	}

	/**
	 * This method initializes saveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSaveButton() {
		if (saveButton == null) {
			saveButton = new JButton();
			saveButton.setBounds(new Rectangle(442, 372, 81, 17));
			saveButton.setText(Locale.get("PaymentWindow.saveButton"));
			saveButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					HashMap<String, Serializable> attrs = new HashMap<String, Serializable>();					

					//Update
					if (getId() != null) attrs.put("reference", (Long) getId());

					Calendar date = Calendar.getInstance();
					date.setTime(getDateTextField().getDate());
					attrs.put("paymentDate", date);

					String status = null;

					switch (getStatusComboBox().getSelectedIndex()) {
					case 0:
						status = "Aberto";
						break;
					case 1:
						status = "Pechado";
						break;					
					}

					attrs.put("status", status);
					attrs.put("invoiceNumber", getInvoiceNumberTextField().getText());					           
					attrs.put("invoiceSerial", getInvoiceSerialTextField().getText());					

					String payMode = null;
					switch (getPayModeComboBox().getSelectedIndex()) {
					case 0:
						payMode = "Contado";
						break;
					case 1:
						payMode = "Tarxeta";
						break;
					case 2:
						payMode = "Bono";
						break;
					}

					attrs.put("payMode", payMode); 
					attrs.put("amount", getAmountTextField().getText().replace(',', '.'));					
					attrs.put("notes", getNotesTextArea().getText());

					attrs.put("user", SecurityHandler.getAuthenticatedLogin());
					attrs.put("cash", null);

					//Can set the invoice status to issued.

					if (getId() == null) { 

						setDialogMessage(Locale.get("JOptionPane.PaymentWindow.saveButton.issueMessage"));
						setDialogTitle(Locale.get("JOptionPane.PaymentWindow.saveButton.issueTitle"));

						Object[] options = {Locale.get("JOptionPane.Yes"), Locale.get("JOptionPane.No")};
						int n = JOptionPane.showOptionDialog(InvoiceWindow.getInstance(),
								getDialogMessage(),
								getDialogTitle(),
								JOptionPane.YES_NO_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);

						if (n == JOptionPane.YES_OPTION) attrs.put("issue", true);			
						else attrs.put("issue", false);

					}

					setId(getAdapter().save(attrs));

					if (!inputErrors()) {										

						setDialogMessage(Locale.get("JOptionPane.PaymentWindow.saveButton.message"));
						setDialogTitle(Locale.get("JOptionPane.PaymentWindow.saveButton.title"));

						Object[] options = {Locale.get("JOptionPane.Ok")};
						int n = JOptionPane.showOptionDialog(InvoiceWindow.getInstance(),
								getDialogMessage(),
								getDialogTitle(),
								JOptionPane.OK_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);

						if (n == JOptionPane.OK_OPTION) {							
							clean();
							update();
							lock();
							getToolsPanel().reset();					
						}																	

					}					 					 

				}

			});
		}
		return saveButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setBounds(new Rectangle(535, 372, 81, 17));
			cancelButton.setText(Locale.get("PaymentWindow.cancelButton"));
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					(PaymentWindow.getInstance()).clean();
					(PaymentWindow.getInstance()).lock();

					getInvoicesList().setEnabled(true);
					getSearchTextField().setEditable(true);

					loadList();

					getPayButton().setEnabled(true);


				}
			});
		}
		return cancelButton;
	}

	/**
	 * This method initializes payButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getPayButton() {
		if (payButton == null) {
			payButton = new JButton();
			payButton.setBounds(new Rectangle(137, 320, 74, 17));
			payButton.setText(Locale.get("PaymentWindow.payButton"));
			payButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					String invoiceId = (String) ((TextItem) getInvoicesList().getSelectedValues()[0]).getId();

					//Parse the id (invoiceNumber/invoiceSerial)

					int separator = invoiceId.lastIndexOf("/");

					String invoiceNumber = invoiceId.substring(0, separator);
					String invoiceSerial = invoiceId.substring(separator + 1, invoiceId.length());

					getInvoiceNumberTextField().setText(invoiceNumber);
					getInvoiceSerialTextField().setText(invoiceSerial);

					((DefaultListModel)getInvoicesList().getModel()).remove(getInvoicesList().getSelectedIndex());
					if (getInvoicesList().getComponentCount() != 0) getInvoicesList().setSelectedIndex(0);

					getDateTextField().setEnabled(true);
					getStatusComboBox().setEnabled(true);
					getPayModeComboBox().setEnabled(true);
					getAmountTextField().setEditable(true);
					getNotesTextArea().setEditable(true);
					getSaveButton().setEnabled(true);
					getCancelButton().setEnabled(true);

					//getSearchTextField().setEditable(false);
					payButton.setEnabled(false);	
					//getInvoicesList().setEnabled(false);

				}
			});
		}
		return payButton;
	}

	/**
	 * This method initializes toolsPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private ToolsPanel getToolsPanel() {
		if (toolsPanel == null) {
			toolsPanel = new ToolsPanel(this, new PaymentListAdapter());
			toolsPanel.setBounds(new Rectangle(225, 400, this.getWidth(), 50));
		}
		return toolsPanel;
	}	

	@Override
	public void cleanErrors() {

		getInfoPanel().setInfoText(getInfoPanel().getInfoText());

		getInvoiceNumberTextField().setBorder(UIManager.getBorder("TextField.border"));
		getInvoiceSerialTextField().setBorder(UIManager.getBorder("TextField.border"));
		getStatusComboBox().setBorder(UIManager.getBorder("ComboBox.border"));

	}

	@Override
	public void lock() {

		getDateTextField().setEnabled(false);
		getStatusComboBox().setEnabled(false);
		getPayModeComboBox().setEnabled(false);
		getAmountTextField().setEditable(false);
		getNotesTextArea().setEditable(false);
		getSaveButton().setEnabled(false);
		getCancelButton().setEnabled(false);
		getPayButton().setEnabled(false);
		getInvoicesList().setEnabled(false);
		getSearchTextField().setEditable(false);

	}

	@Override
	public boolean showErrors() {
		return inputErrors();
	}

	@Override
	public void unlock() {

		if (getId() == null) {
			//Add
			getSearchTextField().setEditable(true);
			getPayButton().setEnabled(true);	
			getInvoicesList().setEnabled(true);

		} else {
			//Edit

			//Closed payment can't be editable
			if (getStatusComboBox().getSelectedIndex() == 1) {

				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("PaymentWindow.errors.update")+ "</html>");

				Toolkit.getDefaultToolkit().beep();

				throw new UnsupportedOperationException();

			}

			getDateTextField().setEnabled(true);
			getPayModeComboBox().setEnabled(true);
			getAmountTextField().setEditable(true);
			getNotesTextArea().setEditable(true);
			getSaveButton().setEnabled(true);			

		}

	}

	@Override
	public void update() {
		loadList();
	}

	@Override
	public void clean() {

		setId(null);
		getReferenceTextField().setText("");
		getDateTextField().setDate((Calendar.getInstance()).getTime());
		getStatusComboBox().setSelectedIndex(0);
		getInvoiceNumberTextField().setText("");
		getInvoiceSerialTextField().setText("");
		getPayModeComboBox().setSelectedIndex(0);
		getAmountTextField().setText("0");
		getNotesTextArea().setText("");		
		getSearchTextField().setText("");

		cleanErrors();

	}

	@Override
	public void fill(HashMap fields) {

		setId((Long) fields.get("id"));
		getReferenceTextField().setText(((Long) fields.get("reference")).toString());		
		getDateTextField().setDate(((Calendar) fields.get("paymentDate")).getTime());

		if (((String) fields.get("status")).equals("Aberto")) getStatusComboBox().setSelectedIndex(0);
		if (((String) fields.get("status")).equals("Pechado")) getStatusComboBox().setSelectedIndex(1);

		getInvoiceNumberTextField().setText(((Long) fields.get("invoiceNumber")).toString());
		getInvoiceSerialTextField().setText((String) fields.get("invoiceSerial"));

		if(((String) fields.get("payMode")).equals("Contado")) getPayModeComboBox().setSelectedIndex(0);
		if(((String) fields.get("payMode")).equals("Tarxeta")) getPayModeComboBox().setSelectedIndex(1);
		if(((String) fields.get("payMode")).equals("Bono")) getPayModeComboBox().setSelectedIndex(2);

		getAmountTextField().setText(((BigDecimal) fields.get("amount")).toString());
		getNotesTextArea().setText((String) fields.get("notes"));		

	}

	private boolean inputErrors() {

		if (!ErrorsHandler.getErrors().isEmpty())  {

			if (ErrorsHandler.getErrorAt(0).getItem().equals("unaceptableInvoice")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("PaymentWindow.errors.unaceptableInvoice")+ "</html>");

				getInvoiceNumberTextField().setBorder(new LineBorder(Color.red, 2));
				getInvoiceSerialTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("issuedInvoice")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("PaymentWindow.errors.issuedInvoice")+ "</html>");

				getInvoiceNumberTextField().setBorder(new LineBorder(Color.red, 2));
				getInvoiceSerialTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("status")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("PaymentWindow.errors.status")+ "</html>");

				getStatusComboBox().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;

			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("update")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("PaymentWindow.errors.update")+ "</html>");

				Toolkit.getDefaultToolkit().beep();
				return true;

			}

		}

		return false;

	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
